@@ -87,7 +87,7 @@ from .binfield import BinField
 from .segysampleformat import SegySampleFormat
 from .tracesortingformat import TraceSortingFormat
 from .tracefield import TraceField
-from .su import su
+from . import su
 from .open import open
 from .create import create
 from .segy import SegyFile, spec
