@@ -3,7 +3,6 @@ import collections
 import segyio
 from .binfield import BinField
 from .tracefield import TraceField
-from .su import keys as sukeys
 
 class Field(collections.MutableMapping):
     """
@@ -33,7 +32,132 @@ class Field(collections.MutableMapping):
                 if  x != TraceField.UnassignedInt1
                 and x != TraceField.UnassignedInt2]
 
-    _kwargs = dict(sukeys)
+    _kwargs = {
+        'tracl' : TraceField.TRACE_SEQUENCE_LINE,
+        'tracr' : TraceField.TRACE_SEQUENCE_FILE,
+        'fldr'  : TraceField.FieldRecord,
+        'tracf' : TraceField.TraceNumber,
+        'ep'    : TraceField.EnergySourcePoint,
+        'cdp'   : TraceField.CDP,
+        'cdpt'  : TraceField.CDP_TRACE,
+        'trid'  : TraceField.TraceIdentificationCode,
+        'nvs'   : TraceField.NSummedTraces,
+        'nhs'   : TraceField.NStackedTraces,
+        'duse'  : TraceField.DataUse,
+        'offset': TraceField.offset,
+        'gelev' : TraceField.ReceiverGroupElevation,
+        'selev' : TraceField.SourceSurfaceElevation,
+        'sdepth': TraceField.SourceDepth,
+        'gdel'  : TraceField.ReceiverDatumElevation,
+        'sdel'  : TraceField.SourceDatumElevation,
+        'swdep' : TraceField.SourceWaterDepth,
+        'gwdep' : TraceField.GroupWaterDepth,
+        'scalel': TraceField.ElevationScalar,
+        'scalco': TraceField.SourceGroupScalar,
+        'sx'    : TraceField.SourceX,
+        'sy'    : TraceField.SourceY,
+        'gx'    : TraceField.GroupX,
+        'gy'    : TraceField.GroupY,
+        'counit': TraceField.CoordinateUnits,
+        'wevel' : TraceField.WeatheringVelocity,
+        'swevel': TraceField.SubWeatheringVelocity,
+        'sut'   : TraceField.SourceUpholeTime,
+        'gut'   : TraceField.GroupUpholeTime,
+        'sstat' : TraceField.SourceStaticCorrection,
+        'gstat' : TraceField.GroupStaticCorrection,
+        'tstat' : TraceField.TotalStaticApplied,
+        'laga'  : TraceField.LagTimeA,
+        'lagb'  : TraceField.LagTimeB,
+        'delrt' : TraceField.DelayRecordingTime,
+        'muts'  : TraceField.MuteTimeStart,
+        'mute'  : TraceField.MuteTimeEND,
+        'ns'    : TraceField.TRACE_SAMPLE_COUNT,
+        'dt'    : TraceField.TRACE_SAMPLE_INTERVAL,
+        'gain'  : TraceField.GainType,
+        'igc'   : TraceField.InstrumentGainConstant,
+        'igi'   : TraceField.InstrumentInitialGain,
+        'corr'  : TraceField.Correlated,
+        'sfs'   : TraceField.SweepFrequencyStart,
+        'sfe'   : TraceField.SweepFrequencyEnd,
+        'slen'  : TraceField.SweepLength,
+        'styp'  : TraceField.SweepType,
+        'stat'  : TraceField.SweepTraceTaperLengthStart,
+        'stae'  : TraceField.SweepTraceTaperLengthEnd,
+        'tatyp' : TraceField.TaperType,
+        'afilf' : TraceField.AliasFilterFrequency,
+        'afils' : TraceField.AliasFilterSlope,
+        'nofilf': TraceField.NotchFilterFrequency,
+        'nofils': TraceField.NotchFilterSlope,
+        'lcf'   : TraceField.LowCutFrequency,
+        'hcf'   : TraceField.HighCutFrequency,
+        'lcs'   : TraceField.LowCutSlope,
+        'hcs'   : TraceField.HighCutSlope,
+        'year'  : TraceField.YearDataRecorded,
+        'day'   : TraceField.DayOfYear,
+        'hour'  : TraceField.HourOfDay,
+        'minute': TraceField.MinuteOfHour,
+        'sec'   : TraceField.SecondOfMinute,
+        'timbas': TraceField.TimeBaseCode,
+        'trwf'  : TraceField.TraceWeightingFactor,
+        'grnors': TraceField.GeophoneGroupNumberRoll1,
+        'grnofr': TraceField.GeophoneGroupNumberFirstTraceOrigField,
+        'grnlof': TraceField.GeophoneGroupNumberLastTraceOrigField,
+        'gaps'  : TraceField.GapSize,
+        'otrav' : TraceField.OverTravel,
+        'cdpx'  : TraceField.CDP_X,
+        'cdpy'  : TraceField.CDP_Y,
+        'iline' : TraceField.INLINE_3D,
+        'xline' : TraceField.CROSSLINE_3D,
+        'sp'    : TraceField.ShotPoint,
+        'scalsp': TraceField.ShotPointScalar,
+        'trunit': TraceField.TraceValueMeasurementUnit,
+        'tdcm'  : TraceField.TransductionConstantMantissa,
+        'tdcp'  : TraceField.TransductionConstantPower,
+        'tdunit': TraceField.TransductionUnit,
+        'triden': TraceField.TraceIdentifier,
+        'sctrh' : TraceField.ScalarTraceHeader,
+        'stype' : TraceField.SourceType,
+        'sedm'  : TraceField.SourceEnergyDirectionMantissa,
+        'sede'  : TraceField.SourceEnergyDirectionExponent,
+        'smm'   : TraceField.SourceMeasurementMantissa,
+        'sme'   : TraceField.SourceMeasurementExponent,
+        'smunit': TraceField.SourceMeasurementUnit,
+        'uint1' : TraceField.UnassignedInt1,
+        'uint2' : TraceField.UnassignedInt2,
+
+        'jobid' : BinField.JobID,
+        'lino'  : BinField.LineNumber,
+        'reno'  : BinField.ReelNumber,
+        'ntrpr' : BinField.Traces,
+        'nart'  : BinField.AuxTraces,
+        'hdt'   : BinField.Interval,
+        'dto'   : BinField.IntervalOriginal,
+        'hns'   : BinField.Samples,
+        'nso'   : BinField.SamplesOriginal,
+        'format': BinField.Format,
+        'fold'  : BinField.EnsembleFold,
+        'tsort' : BinField.SortingCode,
+        'vscode': BinField.VerticalSum,
+        'hsfs'  : BinField.SweepFrequencyStart,
+        'hsfe'  : BinField.SweepFrequencyEnd,
+        'hslen' : BinField.SweepLength,
+        'hstyp' : BinField.Sweep,
+        'schn'  : BinField.SweepChannel,
+        'hstas' : BinField.SweepTaperStart,
+        'hstae' : BinField.SweepTaperEnd,
+        'htatyp': BinField.Taper,
+        'hcorr' : BinField.CorrelatedTraces,
+        'bgrcv' : BinField.BinaryGainRecovery,
+        'rcvm'  : BinField.AmplitudeRecovery,
+        'mfeet' : BinField.MeasurementSystem,
+        'polyt' : BinField.ImpulseSignalPolarity,
+        'vpol'  : BinField.VibratoryPolarity,
+        'unas1' : BinField.Unassigned1,
+        'rev'   : BinField.SEGYRevision,
+        'trflag': BinField.TraceFlag,
+        'exth'  : BinField.ExtendedHeaders,
+        'unas2' : BinField.Unassigned2,
+    }
 
     def __init__(self, buf, kind, traceno = None, filehandle = None, readonly = True):
         # do setup of kind/keys first, so that keys() work. if this method
