@@ -58,6 +58,11 @@ PyObject* ValueError( const char* msg ) {
     return NULL;
 }
 
+template< typename T1 >
+PyObject* ValueError( const char* msg, T1 t1 ) {
+    return PyErr_Format( PyExc_ValueError, msg, t1 );
+}
+
 template< typename T1, typename T2 >
 PyObject* ValueError( const char* msg, T1 t1, T2 t2 ) {
     return PyErr_Format( PyExc_ValueError, msg, t1, t2 );
@@ -222,8 +227,9 @@ namespace fd {
 int init( segyiofd* self, PyObject* args, PyObject* kwargs ) {
     char* filename = NULL;
     char* mode = NULL;
+    int endian = 0;
 
-    if( !PyArg_ParseTuple( args, "ss", &filename, &mode ) )
+    if( !PyArg_ParseTuple( args, "ssi", &filename, &mode, &endian ) )
         return -1;
 
     if( std::strlen( mode ) == 0 ) {
@@ -253,6 +259,23 @@ int init( segyiofd* self, PyObject* args, PyObject* kwargs ) {
         return -1;
     }
 
+    switch( endian ) {
+        case 0:
+        case SEGY_LSB:
+        case SEGY_MSB:
+            break;
+
+        default:
+            ValueError( "internal: unexpected endianness, was %d", endian );
+            return -1;
+    }
+
+    int err = segy_set_format( fd, endian );
+    if( err ) {
+        ValueError( "internal: error setting endianness, was %d", endian );
+        return -1;
+    }
+
     /*
      * init can be called multiple times, which is treated as opening a new
      * file on the same object. That means the previous file handle must be
@@ -421,6 +444,63 @@ PyObject* segycreate( segyiofd* self, PyObject* args, PyObject* kwargs ) {
     return (PyObject*) self;
 }
 
+PyObject* suopen( segyiofd* self, PyObject* args ) {
+    segy_file* fp = self->fd;
+    if( !fp ) return NULL;
+
+    if( !PyArg_ParseTuple( args, "" ) )
+        return NULL;
+
+    int err = segy_set_format( fp, SEGY_IEEE_FLOAT_4_BYTE );
+
+    if( err )
+        return RuntimeError( "internal: unable to set type to IEEE float " );
+
+    char header[ SEGY_TRACE_HEADER_SIZE ] = {};
+
+    err = segy_traceheader( fp, 0, header, 0, 0 );
+    if( err )
+        return IOError( "unable to read first trace header in SU file" );
+
+    int32_t f;
+    segy_get_field( header, SEGY_TR_SAMPLE_COUNT, &f );
+
+    const long trace0 = 0;
+    const int samplecount = f;
+    const int elemsize = sizeof( float );
+    int trace_bsize = elemsize * samplecount;
+
+    int tracecount;
+    err = segy_traces( fp, &tracecount, trace0, trace_bsize );
+    switch( err ) {
+        case SEGY_OK: break;
+
+        case SEGY_FSEEK_ERROR:
+            return IOErrno();
+
+        case SEGY_INVALID_ARGS:
+            return RuntimeError( "unable to count traces, "
+                                 "no data traces past headers" );
+
+        case SEGY_TRACE_SIZE_MISMATCH:
+            return RuntimeError( "trace count inconsistent with file size, "
+                                 "trace lengths possibly of non-uniform" );
+
+        default:
+            return Error( err );
+    }
+
+    self->trace0 = trace0;
+    self->trace_bsize = trace_bsize;
+    self->format = SEGY_IEEE_FLOAT_4_BYTE;
+    self->elemsize = elemsize;
+    self->samplecount = samplecount;
+    self->tracecount = tracecount;
+
+    Py_INCREF( self );
+    return (PyObject*) self;
+}
+
 void dealloc( segyiofd* self ) {
     self->fd.close();
     Py_TYPE( self )->tp_free( (PyObject*) self );
@@ -1217,6 +1297,8 @@ PyMethodDef methods [] = {
     { "segymake", (PyCFunction) fd::segycreate,
       METH_VARARGS | METH_KEYWORDS, "Create file." },
 
+    { "suopen", (PyCFunction) fd::suopen, METH_VARARGS, "Open SU file." },
+
     { "close", (PyCFunction) fd::close, METH_VARARGS, "Close file." },
     { "flush", (PyCFunction) fd::flush, METH_VARARGS, "Flush file." },
     { "mmap",  (PyCFunction) fd::mmap,  METH_NOARGS,  "mmap file."  },
